from telebotify.models.jsonable import Jsonable
from telebotify.models.user import User


class Message(Jsonable):

    def __init__(self,
                 message_id,
                 from_user: User,
                 date,
                 chat,
                 text=None):
        self.message_id = message_id
        self.from_user = from_user
        self.date = date
        self.chat = chat
        self.text = text

    @classmethod
    def from_json(cls, json_dict: dict):
        message_id = json_dict["message_id"]
        from_user = User.from_json(json_dict["from"])
        date = json_dict["date"]
        chat = Chat.from_json(json_dict["chat"])
        text = dict.get(json_dict, "text")
        return cls(message_id,
                   from_user,
                   date,
                   chat,
                   text)



import os

from pi18n import TranslationService
from telebotify.services.http_client import HttpClient
from telebotify.utils.file_utils import get_resource_path

translation_service = TranslationService(get_resource_path('telebotify.resources', 'i18n'), os.environ.get("LOCALE", "es"))



import importlib.resources as pkg_resources
from os import PathLike
from types import ModuleType
from typing import Union


def get_resource_path(package: Union[str, ModuleType], resource: Union[str, PathLike]) -> str:
    with pkg_resources.path(package, resource) as p:
        return p.__str__()



from telebotify.models.jsonable import Jsonable
from telebotify.models.message import Message


class Update(Jsonable):

    def __init__(self,
                 update_id: int = None,
                 message: Message = None,
                 edited_message: Message = None,
                 channel_post: Message = None,
                 edited_channel_post: Message = None,
                 inline_query=None,
                 chosen_inline_result=None,
                 callback_query=None,
                 shipping_query=None,
                 pre_checkout_query=None,
                 poll=None,
                 poll_answer=None,
                 my_chat_member=None,
                 chat_member=None):
        self.update_id = update_id
        self.message = message
        self.edited_message = edited_message
        self.channel_post = channel_post
        self.edited_channel_post = edited_channel_post
        self.inline_query = inline_query
        self.chosen_inline_result = chosen_inline_result
        self.callback_query = callback_query
        self.shipping_query = shipping_query
        self.pre_checkout_query = pre_checkout_query
        self.poll = poll
        self.poll_answer = poll_answer
        self.my_chat_member = my_chat_member
        self.chat_member = chat_member

    @classmethod
    def from_json(cls, json_dict: dict):
        update_id = dict.get(json_dict, "update_id")
        message = None if dict.get(json_dict, "message") is None else Message.from_json(dict.get(json_dict, "message"))
        edited_message = None if dict.get(json_dict, "edited_message") is None else Message.from_json(dict.get(json_dict, "edited_message"))
        channel_post = None if dict.get(json_dict, "channel_post") is None else Message.from_json(dict.get(json_dict, "channel_post"))
        edited_channel_post = None if dict.get(json_dict, "edited_channel_post") is None else Message.from_json(dict.get(json_dict, "edited_channel_post"))
        inline_query = dict.get(json_dict, "inline_query")
        chosen_inline_result = dict.get(json_dict, "chosen_inline_result")
        callback_query = dict.get(json_dict, "callback_query")
        shipping_query = dict.get(json_dict, "shipping_query")
        pre_checkout_query = dict.get(json_dict, "pre_checkout_query")
        poll = dict.get(json_dict, "poll")
        poll_answer = dict.get(json_dict, "poll_answer")
        my_chat_member = dict.get(json_dict, "my_chat_member")
        chat_member = dict.get(json_dict, "chat_member")

        return cls(
                 update_id,
                 message,
                 edited_message,
                 channel_post,
                 edited_channel_post,
                 inline_query,
                 chosen_inline_result,
                 callback_query,
                 shipping_query,
                 pre_checkout_query,
                 poll,
                 poll_answer,
                 my_chat_member,
                 chat_member)



from typing import Dict, Any

import requests as requests

from telebotify.models import Body
from telebotify.utils import ModelMapper


class HttpClient:
    base_url = 'https://api.telegram.org/bot'

    def __init__(self, api_key: str):
        self.base_url += api_key

    def get(self, url, cls):
        r = requests.get(f'{self.base_url}/{url}')
        body: Body = ModelMapper.map(r.json(), Body)
        if body.ok:
            return ModelMapper.map(body.result, cls)
        else:
            raise Exception(r)

    def post(self, url, data: Dict[str, Any] = None, cls=None):
        r = requests.post(f'{self.base_url}/{url}', data)
        body: Body = ModelMapper.map(r.json(), Body)
        if body.ok:
            return ModelMapper.map(body.result, cls) if cls is not None else None
        else:
            print(r.reason)
            raise Exception(r)



from typing import Dict, Callable

from telebotify.models.user import User
from telebotify.services import translation_service, HttpClient


class Bot:
    def __init__(self,
                 api_key: str,
                 webhook_url: str = '',
                 commands: Dict[str, Callable] = {},
                 cert_path: str = None):
        self.http = HttpClient(api_key)
        self.commands = self.set_commands(commands)
        self.set_webhook(webhook_url, cert_path)
        self.last_update_id = 0
        self.started = False

    def get_me(self) -> User:
        return self.http.get('getMe', User)

    def set_webhook(self, webhook_url, certificate):
        data = {"url": webhook_url}
        if certificate is not None:
            data["certificate"] = open(certificate, 'r')
        self.http.post('setWebhook', data)

    def send_message(self, chat_id, text, **kwargs):
        data = {"chat_id": chat_id, "text": text}
        if dict.get(kwargs, "reply_to_message_id") is not None:
            data["reply_to_message_id"] = kwargs["reply_to_message_id"]
        self.http.post('sendMessage', data)

    def set_commands(self, commands):
        def start(message):
            if self.started:
                self.send_message(message.chat.chat_id,
                                  translation_service.get("START_COMMAND_BOT_ALREADY_STARTED_MESSAGE"),
                                  reply_to_message_id=message.message_id)
            else:
                self.started = True
                self.send_message(message,
                                  translation_service.get("START_COMMAND_MESSAGE"),
                                  reply_to_message_id=message.message_id)

        if "start" not in commands:
            commands["start"] = lambda _, message: start(message)

        commands["help"] = lambda _, message: self.send_message(message,
                                                                "Help",
                                                                reply_to_message_id=message.message_id)
        return commands



from telebotify.models.jsonable import Jsonable


class User(Jsonable):

    def __init__(self,
                 user_id=None,
                 is_bot=None,
                 first_name=None,
                 last_name=None,
                 username=None,
                 language_code=None,
                 can_join_groups=None,
                 can_read_all_group_messages=None,
                 supports_inline_queries=None):
        self.user_id = user_id
        self.is_bot = is_bot
        self.first_name = first_name
        self.last_name = last_name
        self.username = username
        self.language_code = language_code
        self.can_join_groups = can_join_groups
        self.can_read_all_group_messages = can_read_all_group_messages
        self.supports_inline_queries = supports_inline_queries

    @classmethod
    def from_json(cls, json_dict: dict):
        user_id = json_dict["id"]
        is_bot = json_dict["is_bot"]
        first_name = json_dict["first_name"]
        last_name = dict.get(json_dict, "last_name")
        username = json_dict["username"]
        language_code = dict.get(json_dict, "language_code")
        can_join_groups = dict.get(json_dict, "can_join_groups")
        can_read_all_group_messages = dict.get(json_dict, "can_read_all_group_messages")
        supports_inline_queries = dict.get(json_dict, "supports_inline_queries")
        return cls(user_id,
                   is_bot,
                   first_name,
                   last_name,
                   username,
                   language_code,
                   can_join_groups,
                   can_read_all_group_messages,
                   supports_inline_queries)



from setuptools import find_packages, setup
from codecs import open
from os import path

HERE = path.abspath(path.dirname(__file__))

with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='telebotify',
    packages=find_packages(),
    version='0.1.8',
    description='A Python library for developing Telegram bots',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Alejandro de Felipe',
    license='MIT',
    include_package_data=True,)



class ModelMapper:

    @staticmethod
    def map(object_dict: dict, cls):
        object_instance = cls()
        attributes = list(vars(object_instance).keys())
        for attribute in attributes:
            try:
                setattr(object_instance, attribute, object_dict[attribute])
            except KeyError:
                setattr(object_instance, attribute, None)
        return object_instance



from telebotify.models.jsonable import Jsonable


class Chat(Jsonable):
    def __init__(self, chat_id, chat_type, title=None, username=None, first_name=None, last_name=None,
                 all_members_are_administrators=None, photo=None, description=None, invite_link=None,
                 pinned_message=None, sticker_set_name=None, can_set_sticker_set=None):
        self.chat_type = chat_type
        self.last_name = last_name
        self.first_name = first_name
        self.username = username
        self.chat_id = chat_id
        self.title = title
        self.all_members_are_administrators = all_members_are_administrators
        self.photo = photo
        self.description = description
        self.invite_link = invite_link
        self.pinned_message = pinned_message
        self.sticker_set_name = sticker_set_name
        self.can_set_sticker_set = can_set_sticker_set

    @classmethod
    def from_json(cls, json_dict: dict):
        chat_type = dict.get(json_dict, "type")
        last_name = dict.get(json_dict, "last_name")
        first_name = dict.get(json_dict, "first_name")
        username = dict.get(json_dict, "username")
        chat_id = dict.get(json_dict, "id")
        title = dict.get(json_dict, "title")
        all_members_are_administrators = dict.get(json_dict, "all_members_are_administrators")
        photo = dict.get(json_dict, "photo")
        description = dict.get(json_dict, "description")
        invite_link = dict.get(json_dict, "invite_link")
        pinned_message = dict.get(json_dict, "pinned_message")
        sticker_set_name = dict.get(json_dict, "sticker_set_name")
        can_set_sticker_set = dict.get(json_dict, "can_set_sticker_set")

        return cls(chat_id,
                   chat_type,
                   title,
                   username,
                   first_name,
                   last_name,
                   all_members_are_administrators,
                   photo,
                   description,
                   invite_link,
                   pinned_message,
                   sticker_set_name,
                   can_set_sticker_set)
